<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\CertisignService;
use Ems\Util\ArrayUtil;
use Ems\Util\Base64Util;
use Ems\Util\ExpectedException;

class CertisignController {
    /** @var CertisignService */
    protected $certisignService;

    public function __construct(
        CertisignService $certisignService
    ) {
        $this->certisignService = $certisignService;
    }

    public function certisignUpload(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $base64 = (string) $row['base64'];
            $fileName = (string) $row['fileName'];

            $uploadId = $this->certisignService->upload(
                $fileName, $base64
            );

            $dataset = [
                'uploadId' => $uploadId,
            ];

            $response->addDataSet(new DataSet('certisignUpload', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignCreate(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $responseBody = $this->certisignService->create(
                $row
            );

            $response->addDataSet(new DataSet('certisignCreate', $responseBody));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignDelete(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $id = (int) $row['id'];

            $this->certisignService->delete($id);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignCallback(
        Request $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $this->certisignService->callback($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignValidateSignatures(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $documentKey = $row['documentKey'];

            $responseBody = $this->certisignService->validateSignatures(
                $documentKey
            );

            $response->addDataSet(new DataSet('certisignValidateSignatures', $responseBody));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveAssinaturaEletronica(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $row = ArrayUtil::setEmptyItemsToNull($row);
            $this->certisignService->saveAssinaturaEletronica($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteAssinaturaEletronica(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrIntegracao = (int) $row['NRINTELETRDIGIT'];

            $this->certisignService->deleteAssinaturaEletronica($nrIntegracao);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function downloadAssinaturaEletronica(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrIntegracao = (int) $row['NRINTELETRDIGIT'];

            $file = $this->certisignService->downloadAssinaturaEletronica($nrIntegracao);

            $response->addDataSet(new DataSet('downloadAssinaturaEletronica', $file));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
