<?php

namespace Ems\Logic\Offline;

class OfflineClockIn extends OfflineEntity {
    
    /* @var string */
    private $nrVinculo;
    /* @var string */
    private $time;
    /* @var string */
    private $date;
    /* @var bool */
    private $isLunchBreak;
    /* @var string */
    private $file;
    /* @var OfflineAbsence[] */
    private $justifiedAbsence;
    
    /* @var int */
    private $nrMarcacao;
    /* @var array */
    private $justifiedAbsenceRow;
    
    private function __construct(?string $nrVinculo, ?string $time, ?string $date, ?bool $isLunchBreak, ?string $file, ?array $justifiedAbsenceRow) {
        $this->nrVinculo = $nrVinculo;
        $this->time = $time;
        $this->date = $date;
        $this->isLunchBreak = $isLunchBreak;
        $this->file = $file;
        $this->justifiedAbsenceRow = $justifiedAbsenceRow;
    }
    
    public static function fromArray(?array $row): OfflineClockIn {
        return new OfflineClockIn(
            $row['nrVinculo'] ?? null,
            $row['time'] ?? null,
            $row['date'] ?? null,
            $row['isLunchBreak'] ?? null,
            $row['file'] ?? null,
            $row['justifiedAbsence'] ?? null
        );
    }
    
    public function synchronize(): void {
        $this->setClockIn();
        $this->synchronizeAbsence();
    }
    
    public function getJustifiedAbsence(): OfflineAbsence {
        return $this->justifiedAbsence;
    }
    
    public function getNrMarcacao(): int {
        return $this->nrMarcacao;
    }
    
    private function setClockIn(): void {
        $nrMarcacao = $this->getMarcacaoServiceInstance()->setMarcacao(
            $this->nrVinculo,
            $this->getNrorg(),
            $this->getCdoperador(),
            null,
            null,
            $this->file,
            $this->getColetorPF(),
            $this->getFullTime()
        );
        $this->nrMarcacao = (int) $nrMarcacao;
    }
    
    private function synchronizeAbsence(): void {
        $this->createAbsence();
        
        if (!is_null($this->justifiedAbsence)) {
            $this->justifiedAbsence->synchronize();
        }
    }
    
    private function createAbsence(): void {
        if (empty($this->justifiedAbsenceRow) && !$this->isLunchBreak) {
            return;
        }
        
        $justifiedAbsenceFullRow = $this->prepareAbsenceRow($this->justifiedAbsenceRow);
        $this->justifiedAbsence = OfflineAbsence::fromArray($justifiedAbsenceFullRow);
    }
    
    private function prepareAbsenceRow(?array $row): ?array {
        $row['nrVinculo'] = $this->nrVinculo;
        $row['nrMarcacao'] = $this->nrMarcacao;
        $row['isLunchBreak'] = $this->isLunchBreak;
        $row['date'] = $this->date;
        return $row;
    }
    
    private function getFullTime(): string {
        $fullDate = $this->date . ' ' . $this->time . ':00';
        return $fullDate;
    }
    
    private function getColetorPF(): ?int {
        return $this->getParamOrgServiceInstance()->getColetorPF();
    }
}